@@ -1,13 +1,15 @@
 
 import sys
 import os
+import shutil
 import glob
 import itertools
 import warnings
 import numpy as np
+from numpy.lib.recfunctions import rename_fields as np_rename_fields
 from scipy.interpolate import griddata
 from shapely import wkt
-from shapely.geometry import Point, Polygon, MultiPoint, MultiPolygon, LineString, MultiLineString, LinearRing, mapping, shape
+from shapely.geometry import Point, Polygon, MultiPoint, MultiPolygon, LineString, MultiLineString, LinearRing, box, mapping, shape
 from shapely.ops import transform, linemerge, unary_union, polygonize
 from shapely.affinity import translate
 import pandas as pd
@@ -38,17 +40,21 @@ import jenkspy
 
 import zipfile
 import tarfile
+import gzip
+import netCDF4 as nc
 import utm
 import numbers
 import osmnx as osm
 import folium
+from xarray.core import variable
 
 from lfmaptools.utilities import (_nice_round, 
                                   _represents_int, 
                                   latlon_to_utm_epsg,
                                   _path_of_file_in_zip,
                                   csv_active_cols,
-                                  csv_num_cols)
+                                  csv_num_cols,
+                                  _decompress_gz)
 from lfmaptools.geotools import LatLon_TM_polygonize, LatLon_TM
 from lfmaptools.epsg_defs import webmerc, wgs84
 from lfmaptools.mapping import add_stamen_basemap
@@ -79,6 +85,15 @@ _LFoutput_fmt = [
     {'file': 'MaxSpeeds.txt', 'column': 5, 'short_name': 'maximum_speed', 'long_name': 'Maximum speed of lahar', 'symbol': 'maxspd', 'units': 'm/s', 'dtype': 'real', 'infile': True, },
     {'file': 'MaxSpeeds.txt', 'column': 6, 'short_name': 'time_of_maximum', 'long_name': 'Time of the maximum speed', 'symbol': 't_maxspd', 'units': 's', 'dtype': 'real', 'infile': True, },
     {'file': 'InundationTime.txt', 'column': 5, 'short_name': 'inundation_time', 'long_name': 'Time of first inundation', 'symbol': 't_inun', 'units': 's', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'maximum_depth', 'long_name': 'Maximum depth of lahar', 'symbol': 'maxh', 'units': 'm', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'time_of_max_depth', 'long_name': 'Time of the maximum depth', 'symbol': 't_maxh', 'units': 's', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'maximum_speed', 'long_name': 'Maximum speed of lahar', 'symbol': 'maxspd', 'units': 'm/s', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'time_of_max_erosion', 'long_name': 'Time of the maximum erosion depth', 'symbol': 't_maxe', 'units': 's', 'dtype': 'real', 'infile': True, },{'file': 'Maximums.nc', 'short_name': 'time_of_max_speed', 'long_name': 'Time of the maximum speed', 'symbol': 't_maxspd', 'units': 's', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'maximum_erosion', 'long_name': 'Maximum erosion depth', 'symbol': 'maxe', 'units': 'm', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'time_of_max_erosion', 'long_name': 'Time of the maximum erosion depth', 'symbol': 't_maxe', 'units': 's', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'maximum_concentration', 'long_name': 'Maximum concentration', 'symbol': 'maxc', 'units': '', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'time_of_max_concentration', 'long_name': 'Time of the maximum concentration', 'symbol': 't_maxc', 'units': 's', 'dtype': 'real', 'infile': True, },
+    {'file': 'Maximums.nc', 'short_name': 'inundation_time', 'long_name': 'Time of first inundation', 'symbol': 't_inun', 'units': 's', 'dtype': 'real', 'infile': True, },
 ]
 _LFoutput_shortnames = [d['short_name'] for d in _LFoutput_fmt]
 def _get_LF_output_fmt(short_name, filetype='snapshot'):
@@ -210,11 +225,14 @@ class LaharFlowData(object):
             self.files=[]
             self.txt_files=[]
             self.tar_files=[]
+            self.nc_files=[]
+            self.gz_files=[]
             
             self.get_files()
             self.get_txt_files()
             self.get_nc_files()
             self.get_tar_files()
+            self.get_gz_files()
             self.get_tiff_files()
 
             self.clatlong = [self.clat,self.clong]
@@ -229,6 +247,7 @@ class LaharFlowData(object):
                 self._get_parameters_from_runinfo()
             self._get_max_depth()
             self._get_max_speed()
+            self._get_max_bed_evolution()
     
     def _get_parameters_from_inputfile(self):
         if self.zipped:
@@ -461,6 +480,9 @@ class LaharFlowData(object):
         lat_res = North_step[0] - clat
         lon_res = East_step[1] - clong
 
+        # if clat<0:
+        #     lat_res = -lat_res
+
         return (lat_res, lon_res)
 
     def get_XY(self):
@@ -484,13 +506,19 @@ class LaharFlowData(object):
     def get_nc_files(self):
         self.nc_files = [f for f in self.files if f.endswith('.nc')]
         self.nc_files.sort()
+    
+    def get_gz_files(self):
+        self.gz_files = [f for f in self.files if f.endswith('.gz')]
+        self.gz_files.sort()
 
     def get_txt_files(self):
         self.txt_files = [f for f in self.files if f.endswith('.txt')]
         self.txt_files.sort()
     
     def get_snapshot_files(self):
-        sfiles = [f for f in self.txt_files if f.strip('.txt').isnumeric()]
+        txt_files = [f.strip('.txt') for f in self.txt_files if f.strip('.txt').isnumeric()]
+        nc_files = [f.strip('.nc') for f in self.nc_files if f.strip('.nc').isnumeric()]
+        sfiles = list(set(txt_files + nc_files))
         sfiles.sort()
         return sfiles
 
@@ -522,12 +550,14 @@ class LaharFlowData(object):
             if fileIn not in self.files:
                 raise ValueError('Result file {file} is not in directory {dir}'.format(file=fileIn, dir=self.dir))
         elif isinstance(fileIn, int):
-            if '{0:06d}.txt'.format(fileIn) in self.files:
+            if '{0:06d}.nc'.format(fileIn) in self.files:
+                fileIn = '{0:06d}.nc'.format(fileIn)
+            elif '{0:06d}.txt'.format(fileIn) in self.files:
                 fileIn = '{0:06d}.txt'.format(fileIn)
             elif '{0:06d}.txt.tar.gz'.format(fileIn) in self.files:
                 fileIn = '{0:06d}.txt.tar.gz'.format(fileIn)
-            elif '{0:06d}.nc'.format(fileIn) in self.files:
-                fileIn = '{0:06d}.nc'.format(fileIn)
+            elif '{0:06d}.nc.gz'.format(fileIn) in self.files:
+                fileIn = '{0:06d}.nc.gz'.format(fileIn)
             else:
                 raise ValueError('Result file {file} is not in directory {dir}'.format(file=fileIn, dir=self.dir))
         else:
@@ -541,6 +571,8 @@ class LaharFlowData(object):
                 filetype = 'MaxHeights.txt'
             elif filename=='MaxSpeeds.txt':
                 filetype = 'MaxSpeeds.txt'
+            if filename=='Maximums.nc':
+                filetype = 'Maximums.nc'
             else:
                 filetype = 'snapshot'
         elif isinstance(filename, int):
@@ -553,6 +585,7 @@ class LaharFlowData(object):
             raise RuntimeError('variable {var} not recognized'.format(var=variable))
 
     def _extract_file(self, filename):
+
         if self.zipped:
             LaharFlowFilePath = _path_of_file_in_zip(filename, self._zipref)[0]
             LaharFlowFile = self._zipref.extract(LaharFlowFilePath, path=self.dir)
@@ -560,19 +593,29 @@ class LaharFlowData(object):
         elif filename in self.txt_files:
             LaharFlowFile = os.path.join(self.dir, filename)
             ExtractPath = self.dir
-        elif filename in self.tar_files:
-            LaharFlowFile = os.path.join(self.dir, filename)
-            ExtractPath = self.dir
         elif filename in self.nc_files:
             LaharFlowFile = os.path.join(self.dir, filename)
             ExtractPath = self.dir
+        elif filename in self.tar_files:
+            LaharFlowFile = os.path.join(self.dir, filename)
+            ExtractPath = self.dir
+        elif filename in self.gz_files:
+            LaharFlowFile = os.path.join(self.dir, filename)
+            ExtractPath = self.dir
+        
+        fname = os.path.split(LaharFlowFile)[1]
 
-        if (os.path.split(LaharFlowFile)[1] in self.tar_files) and (os.path.split(LaharFlowFile)[1] not in self.txt_files):
+        if (fname in self.tar_files) and (fname not in self.txt_files):
             tar = tarfile.open(LaharFlowFile, mode='r')
             tar.extractall(path=ExtractPath)
             tar.close()
             self.get_txt_files()
             LaharFlowFile = '.'.join(LaharFlowFile.split('.')[:-2])
+        elif (fname in self.gz_files) and (fname not in self.txt_files) and (fname not in self.nc_files):
+            LaharFlowFile = _decompress_gz(LaharFlowFile)
+            self.get_txt_files()
+            self.get_nc_files()
+
         return LaharFlowFile
 
     def _get_data_column_names(self, filename):
@@ -582,6 +625,74 @@ class LaharFlowData(object):
         cols = csv_active_cols(LaharFlowFile)
         return cols
 
+    def _get_data_txt(self, LaharFlowFile):
+        try:
+            cols = csv_active_cols(LaharFlowFile)
+            data = np.genfromtxt(LaharFlowFile, delimiter=',', names=True, usecols=cols)
+            return data
+        except:
+            raise RuntimeWarning('Unable to get data from {}'.format(filename))
+    
+    def _get_data_nc(self, LaharFlowFile):
+        rgrp = nc.Dataset(LaharFlowFile, 'r')
+
+        tiles = list(rgrp.groups)
+        nc_variables = list(rgrp[tiles[0]].variables)
+
+        nX = self.nXpertile
+        nY = self.nYpertile
+        nXY = nX*nY
+        nrows = len(tiles)*nXY
+
+        variables = nc_variables.copy()
+
+        dt = [('tile',int)]
+        for v in variables:
+            if v=='x':
+                vname = 'x_distance'
+            elif v=='y':
+                vname = 'y_distance'
+            else:
+                vname = v
+            dt.append((vname, rgrp[tiles[0]][v].dtype))
+
+        data = np.empty((nrows,), dtype=dt)
+        
+        #data[:] = np.nan
+
+        if 'x' in variables:
+            variables.remove('x')
+        if 'y' in variables:
+            variables.remove('y')
+
+        if 'x_distance' in variables:
+            variables.remove('x_distance')
+        if 'y_distance' in variables:
+            variables.remove('y_distance')
+
+        for i, t in enumerate(tiles):
+            grp = rgrp[t]
+            if 'x' in nc_variables:
+                x = grp['x'][:]
+            elif 'x_distance' in nc_variables:
+                x = grp['x_distance'][:]
+            if 'y' in nc_variables:
+                y = grp['y'][:]
+            elif 'y_distance' in nc_variables:
+                y = grp['y_distance'][:]
+            Y, X = np.meshgrid(y,x)
+            data['tile'][i*nXY:(i+1)*nXY] = int(t)
+            data['x_distance'][i*nXY:(i+1)*nXY] = X.flatten()
+            data['y_distance'][i*nXY:(i+1)*nXY] = Y.flatten()
+            for v in variables:
+                data[v][i*nXY:(i+1)*nXY] = grp[v][:].flatten()
+        
+        rgrp.close()
+
+        data = np_rename_fields(data, {'lat':'latitude', 'lon':'longitude'})
+
+        return data
+
     def _get_data(self, filename):
 
         filename = self._valid_input_file(filename)
@@ -591,26 +702,24 @@ class LaharFlowData(object):
         fileext = os.path.splitext(LaharFlowFile)[1]
 
         if fileext=='.txt':
-            try:
-                cols = csv_active_cols(LaharFlowFile)
-                data = np.genfromtxt(LaharFlowFile, delimiter=',', names=True, usecols=cols)
-                return data
-            except:
-                raise RuntimeWarning('Unable to get data from {}'.format(filename))
+            return self._get_data_txt(LaharFlowFile)
         elif fileext=='.nc':
-            print ('Make nc work')
-
-
+            return self._get_data_nc(LaharFlowFile)
+        else:
+            raise ValueError('File {} not recognized'.format(LaharFlowFile))
+        
     
     def raster(self, filename, var=None, full_domain=False, nodata=np.nan, masked=True, crs=None, res=None):
-        filetype = self._file_type(filename)
         
+        filetype = self._file_type(filename)
         filename = self._valid_input_file(filename)
         
-        LaharFlowFile = self._extract_file(filename)
+        # LaharFlowFile = self._extract_file(filename)
+        # print (LaharFlowFile)
         
-        cols = csv_active_cols(LaharFlowFile)
-        data = np.genfromtxt(LaharFlowFile, delimiter=',', names=True, usecols=cols)
+        # cols = csv_active_cols(LaharFlowFile)
+        # data = np.genfromtxt(LaharFlowFile, delimiter=',', names=True, usecols=cols)
+        data = self._get_data(filename)
         
         extra_cols = ['tile', 'x_distance', 'y_distance', 'latitude', 'longitude']
         if var is None:
@@ -634,7 +743,6 @@ class LaharFlowData(object):
         tiles = data['tile'].astype(int)
         uTiles = np.unique(tiles)
 
-
         lats = data['latitude']
         lons = data['longitude']
 
@@ -667,14 +775,15 @@ class LaharFlowData(object):
         ds['longitude'].attrs["units"] = "degrees"
         ds.rio.write_crs(wgs84, inplace=True)
 
-        raster = ds
+        if crs is not None and crs is not wgs84:
+            raster = ds.rio.reproject(crs)
+        else:
+            raster = ds
 
         return raster
 
         # tile_groups = self._connected_tiles(uTiles)
 
-        
-
         # raster = [xa.Dataset()]*len(tile_groups)
 
         # for i, g in enumerate(tile_groups):
@@ -793,7 +902,7 @@ class LaharFlowData(object):
         #     ds['concentration'].attrs["units"] = fmt['units']
         #     ds['concentration'].attrs["symbol"] = fmt['symbol']
 
-        return raster
+        # return raster
 
     def _raster_flatten(raster):
         rflat = raster.data.flatten()
@@ -842,23 +951,18 @@ class LaharFlowData(object):
         return array
 
     def _get_max_bed_evolution(self):
-        last_outfile = self.get_snapshot_files()[-1]
-        
-        dataFile = self._extract_file(last_outfile)
+        last_outfile = int(self.get_snapshot_files()[-1])
 
-        if not os.path.isfile(dataFile):
-            MaxErosion = np.nan
-            MaxDeposit = np.nan
-        else:
-            x, y, bt = self._get_data_column(dataFile, 14)
-            
-            MaxErosion = max(-np.amin(bt),0.0)
-            MaxDeposit = max(np.amax(bt),0.0)
+        data = self.raster(last_outfile, var=['elevation_change'])
+
+        bt = data['elevation_change'].values
+        
+        MaxErosion = max(-np.nanmin(bt),0.0)
+        MaxDeposit = max(np.nanmax(bt),0.0)
 
         self.MaxErosion = MaxErosion
         self.MaxDeposit = MaxDeposit
-
-        return {'MaxErosion': MaxErosion, 'MaxDeposit': MaxDeposit}
+        return
     
     def max_bed_evolution(self, data=None, vmin=None, vmax=None):
         last_outfile = self.get_snapshot_files()[-1]
@@ -880,26 +984,27 @@ class LaharFlowData(object):
 
     def _get_max_depth(self):
         
-        if 'MaxHeights.txt' in self.txt_files:
+        if 'Maximums.nc' in self.nc_files:
+            d = self._get_data('Maximums.nc')
+            self.MaxDepth = d['maximum_depth'].max()
+            return
+        elif 'MaxHeights.txt' in self.txt_files:
             _, _, h = self._get_data_column('MaxHeights.txt', 5)
             self.MaxDepth = np.amax(h)
             return
-        elif 'Maximums.nc' in self.nc_files:
-            # TODO: Make this work
-            return
         else:
-            raise RuntimeWarning('No data to determine maximum depth')
+            print ('make this work')
+            #raise RuntimeWarning('No data to determine maximum depth')
+        return
     
     def max_depth(self, data=None, vmin=None):
 
         if data is None:
-            if 'MaxHeights.txt' in self.txt_files:
+            if 'Maximums.nc' in self.nc_files:
+                data = self.raster('Maximums.nc')
+            elif 'MaxHeights.txt' in self.txt_files:
                 data = self.raster('MaxHeights.txt')
-            elif 'Maximums.nc' in self.nc_files:
-                # TODO: Make this work
-                print('problem')
-                return
-        
+            
         if 'maximum_depth' not in list(data.keys()):
             raise RuntimeError('maximum_depth not in data')
         
@@ -912,26 +1017,26 @@ class LaharFlowData(object):
 
     def _get_max_speed(self):
         
-        if 'MaxSpeeds.txt' in self.txt_files:
+        if 'Maximums.nc' in self.nc_files:
+            d = self._get_data('Maximums.nc')
+            self.MaxSpeed = d['maximum_speed'].max()
+            return
+        elif 'MaxSpeeds.txt' in self.txt_files:
             _, _, spd = self._get_data_column('MaxSpeeds.txt', 5)
             self.MaxSpeed = np.amax(spd)
             return
-        elif 'Maximums.nc' in self.nc_files:
-            # TODO: Make this work
-            return
         else:
-            raise RuntimeWarning('No data to determine maximum speed')
+            print ('make this work')
+            #raise RuntimeWarning('No data to determine maximum speed')
         
     def max_speed(self, data=None, vmin=None):
 
         if data is None:
-            if 'MaxSpeeds.txt' in self.txt_files:
+            if 'Maximums.nc' in self.nc_files:
+                data = self.raster('Maximums.nc')
+            elif 'MaxSpeeds.txt' in self.txt_files:
                 data = self.raster('MaxSpeeds.txt')
-            elif 'Maximums.nc' in self.nc_files:
-                # TODO: Make this work
-                print('problem')
-                return
-        
+            
         if 'maximum_speed' not in list(data.keys()):
             raise RuntimeError('maximum_depth not in data')
 
@@ -945,6 +1050,7 @@ class LaharFlowData(object):
     def inundation_time(self, data=None, vmin=None):
 
         if data is None:
+
             data = self.raster('InundationTime.txt')
         if 'inundation_time' not in list(data.keys()):
             data = self.raster('InundationTime.txt')
@@ -995,22 +1101,36 @@ class LaharFlowData(object):
             dat = dat.where(dat.data>vmin)
 
         return dat
-        
-    def _raster_contour(self, raster, name, cntrs):
+    
+    def clip_raster(self, raster, clip_gdf):
+        # Clip raster data to geodataframe clip_gdf
 
-        data = raster.data
+        (l, b, r, t) = raster.rio.bounds()
+        bbox = gpd.GeoDataFrame(geometry=[box(l,b,r,t)], crs=raster.rio.crs)
+
+        clip_gdf = clip_gdf.to_crs(raster.rio.crs)
+
+        clip_poly = clip_gdf.intersection(bbox)
+
+        try:
+            data_clipped = raster.rio.clip(clip_poly)
+        except:
+            return None
+
+        return data_clipped
+        
+    def _raster_contour(self, raster, var, cntrs):
+
+        data = raster[var].values
         crs = raster.rio.crs
-        x = raster.x.data
-        y = raster.y.data
+        lon = raster.longitude.data
+        lat = raster.latitude.data
 
-        x0, y0 = self.get_XY()
-        x0 += self.utm[0]
-        y0 += self.utm[1]
+        dlon, dlat = raster.rio.resolution()
         
-        fx = interp1d(np.arange(0,len(x)),x)
-        fy = interp1d(np.arange(0,len(y)),y)
+        fx = interp1d(np.arange(0,len(lon)),lon)
+        fy = interp1d(np.arange(0,len(lat)),lat)
 
-        
         for kk, this_cntr in enumerate(cntrs):
             
             new_set = True
@@ -1041,10 +1161,10 @@ class LaharFlowData(object):
                             geom = geom.symmetric_difference(geom1)
                             
             g_tmp = gpd.GeoDataFrame(columns=['contour', 'name', 'geometry'],
-                                 crs=self.utmCode)
+                                 crs=wgs84)
             g_tmp.loc[0, 'contour'] = this_cntr
             g_tmp.loc[[0], 'geometry'] = gpd.GeoSeries(geom)
-            g_tmp.loc[0, 'name'] = name
+            g_tmp.loc[0, 'name'] = var
                 
             if kk==0:
                 g = gpd.GeoDataFrame(g_tmp).set_geometry('geometry')
@@ -1125,7 +1245,7 @@ class LaharFlowData(object):
         else:
             return None
     
-    def POI_value(self,  x, y, data=None, filename=None, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
+    def POI_value(self,  lon, lat, data=None, filename=None, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
         
         if data is None and filename is None:
             raise RuntimeError('cannot have both data=None and filename=None')
@@ -1133,6 +1253,8 @@ class LaharFlowData(object):
         if data is not None and filename is not None:
             raise RuntimeError('cannot give both data and filename')
 
+        assert(var is not None), 'In POI_value, must give variable to extract as named argument var'
+
         if not isinstance(var, list):
             var = [var]
 
@@ -1145,7 +1267,7 @@ class LaharFlowData(object):
         if fill_value is not None:
             data = data.fillna(fill_value)
         
-        value = data.interp(x=x, y=y, method=interp_method)
+        value = data.interp(longitude=lon, latitude=lat, method=interp_method)
 
         val = dict()
         vars = var if var is not None else list(data.keys())
@@ -1154,6 +1276,47 @@ class LaharFlowData(object):
         
         return val
     
+    def POI_buffer(self,  lon, lat, buffer, data=None, filename=None, var=None,
+                   full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear',
+                   name='POI'):
+        
+        if data is None and filename is None:
+            raise RuntimeError('cannot have both data=None and filename=None')
+        
+        if data is not None and filename is not None:
+            raise RuntimeError('cannot give both data and filename')
+
+        assert(var is not None), 'In POI_value, must give variable to extract as named argument var'
+
+        if not isinstance(var, list):
+            var = [var]
+
+        if filename is not None:
+            data = self.raster(filename, var=var, full_domain=full_domain, nodata=np.nan, masked=True, crs=None)
+        
+        data_utm = data.rio.reproject(self.utmCode)
+
+        df = pd.DataFrame({'Name':name, 'Longitude':[lon], 'Latitude':[lat]})
+        gdf = gpd.GeoDataFrame(df, geometry=gpd.points_from_xy(df.Longitude, df.Latitude), crs=wgs84)
+
+        gdf = gdf.to_crs(self.utmCode)
+        gdf['geometry'] = gdf.geometry.buffer(buffer)
+
+        (l, b, r, t) = data_utm.rio.bounds()
+        bbox = gpd.GeoDataFrame(geometry=[box(l,b,r,t)], crs=self.utmCode)
+
+        if not gdf.within(bbox)[0]:
+            return nodata
+
+        clip_poly = gdf.intersection(bbox)
+
+        if fill_value is not None:
+            data_utm = data_utm.fillna(fill_value)
+
+        data_clipped = data_utm.rio.clip(clip_poly)
+        
+        return data_clipped
+    
     def transect_value(self, x, y, filename=None, data=None, var=None, full_domain=False, nodata=np.nan, fill_value=0, interp_method='linear'):
         
         if data is None and filename is None:
